unit uFilemanager;
interface
uses windows, shellapi, uFunctions;
const
  faReadOnly  = $00000001;
  faHidden    = $00000002;
  faSysFile   = $00000004;
  faVolumeID  = $00000008;
  faDirectory = $00000010;
  faArchive   = $00000020;
  faAnyFile   = $0000003F;
Type
  TFileName = type string;
  TSearchRec = record
    Time: Integer;
    Size: Integer;
    Attr: Integer;
    Name: TFileName;
    ExcludeAttr: Integer;
    FindHandle: THandle;
    FindData: TWin32FindData;
  end;
  LongRec = packed record
    case Integer of
      0: (Lo, Hi: Word);
      1: (Words: array [0..1] of Word);
      2: (Bytes: array [0..3] of Byte);
  end;
function GetDrives: string;
function ListFileDir(Path: string; enumDirs:Boolean):String;

implementation

function GetFileType(sFileName: String): String;
var
  FileInfo: TSHFileInfo;
begin
  FillChar(FileInfo, SizeOf(FileInfo), #0);
  SHGetFileInfo(PChar(sFileName), 0, FileInfo, SizeOf(FileInfo),SHGFI_TYPENAME);
  Result := FileInfo.szTypeName;
end;

function FindMatchingFile(var F: TSearchRec): Integer;
var
  LocalFileTime: TFileTime;
begin
  with F do
  begin
    while FindData.dwFileAttributes and ExcludeAttr <> 0 do
      if not FindNextFile(FindHandle, FindData) then
      begin
        Result := GetLastError;
        Exit;
      end;
    FileTimeToLocalFileTime(FindData.ftLastWriteTime, LocalFileTime);
    FileTimeToDosDateTime(LocalFileTime, LongRec(Time).Hi, LongRec(Time).Lo);
    Size := FindData.nFileSizeLow;
    Attr := FindData.dwFileAttributes;
    Name := FindData.cFileName;
  end;
  Result := 0;
end;

procedure FindClose(var F: TSearchRec);
begin
  if F.FindHandle <> INVALID_HANDLE_VALUE then
  begin
    Windows.FindClose(F.FindHandle);
    F.FindHandle := INVALID_HANDLE_VALUE;
  end;
end;

function FindFirst(const Path: string; Attr: Integer;
  var  F: TSearchRec): Integer;
const
  faSpecial = faHidden or faSysFile or faVolumeID or faDirectory;
begin
  F.ExcludeAttr := not Attr and faSpecial;
  F.FindHandle := FindFirstFile(PChar(Path), F.FindData);
  if F.FindHandle <> INVALID_HANDLE_VALUE then
  begin
    Result := FindMatchingFile(F);
    if Result <> 0 then FindClose(F);
  end else
    Result := GetLastError;
end;

function FindNext(var F: TSearchRec): Integer;
begin
  if FindNextFile(F.FindHandle, F.FindData) then
    Result := FindMatchingFile(F) else
    Result := GetLastError;
end;

function ListFileDir(Path: string; enumDirs:Boolean):String;
var
  SR: TSearchRec;
  sTemp:string;
begin
  Result := '';
  If (Path = '') Then Exit;
  If (Path[Length(Path)] <> '\') Then Path := Path + '\';
  if FindFirst(Path + '*.*', faAnyFile, SR) = 0 then
  begin
    repeat
      if enumDirs then begin
        if ((SR.Attr And faDirectory) = faDirectory) then
          If (SR.Name <> '..') and (SR.Name <> '.') then
            Result := Result + SR.Name + '#' + '-|';
      end else begin
        if ((SR.Attr And faDirectory) <> faDirectory) then begin
          sTemp := GetFileType(Path + SR.Name);
          Result := Result + SR.Name + '#' + sTemp + '#' + Inttostr(SR.Size) + '|';
        end;
      end;
    until FindNext(SR) <> 0;
    FindClose(SR);
  end else
    Result := 'Error';
end;

function GetDrives: string;
var
  r: LongWord;
  Drives: array[0..128] of char;
  pDrive: pchar;
begin
  Result := '';
  r := GetLogicalDriveStrings(sizeof(Drives), Drives);
  if r = 0 then exit;
  pDrive := Drives;

  while pDrive^ <> #0 do begin
    result := result + pdrive;
    case GetDriveType(pdrive) of
      DRIVE_REMOVABLE:Result := Result + '#2|';
      DRIVE_FIXED:Result := Result + '#1|';
      DRIVE_REMOTE:Result := Result + '#3|';
      DRIVE_CDROM:Result := Result + '#4|';
    else Result := Result + '#1|';
    end;
    inc(pDrive, 4);
  end;
end;
end.
